"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectFromSource = exports.collect = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const gitignore_parser_1 = __importDefault(require("gitignore-parser"));
const utils_1 = require("../core/utils");
const util_1 = __importDefault(require("util"));
function collect(source, forceLocale) {
    source = source
        ? path_1.default.join(process.cwd(), source)
        : process.cwd();
    // parse gitignore
    // todo: parse .gitignore files in nested directories.
    let ignore;
    const gitIgnorePath = path_1.default.join(source, '.gitignore');
    if (fs_1.default.existsSync(gitIgnorePath)) {
        const content = fs_1.default.readFileSync(gitIgnorePath, 'utf8');
        ignore = gitignore_parser_1.default.compile(content);
    }
    else {
        ignore = gitignore_parser_1.default.compile('');
    }
    function* walk(dirRelative) {
        try {
            const files = fs_1.default.readdirSync(path_1.default.join(source, dirRelative));
            for (const file of files) {
                const filepath = path_1.default.join(source, dirRelative, file);
                const fileRelative = path_1.default.join(dirRelative, file);
                if (ignore.denies(fileRelative)) {
                    continue;
                }
                const stats = fs_1.default.statSync(filepath);
                if (stats.isDirectory()) {
                    yield* walk(fileRelative);
                }
                else {
                    if (ignore.accepts(fileRelative)) {
                        yield fileRelative;
                    }
                }
            }
        }
        catch (e) {
            console.error('Failed to read directory ' + dirRelative);
        }
    }
    const ids = new Map();
    const all = [];
    let defaultLocale;
    let fcount = 0;
    for (const f of walk('')) {
        const ext = f.toLowerCase();
        if (!ext.endsWith('.js') && !ext.endsWith('.ts') && !ext.endsWith('.jsx') && !ext.endsWith('.tsx')) {
            continue;
        }
        fcount++;
        try {
            const content = fs_1.default.readFileSync(path_1.default.join(source, f), 'utf8');
            // === try to find setDefaultLocale()
            if (!forceLocale) {
                const defLoc = /\bsetDefaultLocale\s*\(\s*('|")([a-zA-Z\-]+)('|")\s*\)/.exec(content);
                const newDefLoc = defLoc === null || defLoc === void 0 ? void 0 : defLoc[2];
                if (newDefLoc) {
                    if (defaultLocale && defaultLocale.val !== newDefLoc) {
                        console.error(`Ambiguous default locale. Found '${newDefLoc}' in ${f}, but was defined already defined as '${defaultLocale.val}' in ${defaultLocale.file}. Please use --locale:${defaultLocale.val}`);
                    }
                    defaultLocale = {
                        file: f,
                        val: newDefLoc,
                    };
                }
            }
            collectFromSource(content, ids, all, f);
        }
        catch (e) {
            console.error('Failed to read file ' + f + ': ' + util_1.default.inspect(e));
        }
    }
    // === WRITE RESULT ===
    const result = {};
    for (const v of all) {
        result[v.id] = {
            target: v.source,
        };
    }
    const targetLanguage = (defaultLocale === null || defaultLocale === void 0 ? void 0 : defaultLocale.val) || forceLocale;
    if (!targetLanguage) {
        console.error('Cannot infer the default locale. Either call setDefaultLocale() in analyzed sources, or set the --defaultLocale argument');
        process.exit(1);
    }
    const collected = {
        resources: result,
        targetLanguage,
    };
    return { collected, files: fcount };
}
exports.collect = collect;
function collectFromSource(content, ids, all, f) {
    // === parse loc() calls
    const re = /\bloc\s*(\(\s*('|")([a-zA-Z0-9\-_\s\.]+)('|")\s*(?:,\s*)?\))?\s*`([^`]*)`/g;
    let m;
    while (m = re.exec(content)) {
        let id = m[3];
        const val = m[5];
        // parse format
        const reFormat = /\$\{([^\}]+)\}/g;
        let fm;
        let index = 0;
        let cnt = 0;
        const parts = [];
        const idParts = id ? null : [];
        while (fm = reFormat.exec(val)) {
            const literal = val.substr(index, fm.index - index);
            parts.push(literal);
            idParts === null || idParts === void 0 ? void 0 : idParts.push(literal);
            index = fm.index + fm[0].length;
            parts.push('{', cnt.toString(), '}');
            cnt++;
        }
        // add trailing
        idParts === null || idParts === void 0 ? void 0 : idParts.push(val.substr(index));
        parts.push(val.substr(index));
        const raw = parts.join('');
        id = id || utils_1.autoGenerateId(idParts);
        const exists = ids.get(id);
        // check no dupplicate
        if (exists && exists.source !== raw) {
            if (f === exists.file) {
                console.error(`Duplicate translation id '${id}' found in ${f} but was already defined in ${exists.file}`);
            }
            else {
                console.error(`Duplicate translation id '${id}' found in ${f}`);
            }
            continue;
        }
        const found = {
            file: f,
            id,
            line: 0,
            source: raw,
        };
        all.push(found);
        ids.set(id, found);
    }
}
exports.collectFromSource = collectFromSource;
//# sourceMappingURL=collect.js.map