"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.translateInContext = exports.translateObject = exports.toJsonStorable = exports.toLocalizable = exports.jsonParseLocalized = void 0;
const smartloc_1 = require("./smartloc");
const moment_1 = __importDefault(require("moment"));
const literal_1 = require("./literal");
/**
 * Parse a JSON string where localized strings have been serialized in their non translated form
 */
function jsonParseLocalized(value) {
    const parsed = JSON.parse(value);
    return _toLocalizable(parsed, 50);
}
exports.jsonParseLocalized = jsonParseLocalized;
/**
 * Transforms an object that has been deserialized from a non translated from to an object that is translatable
 */
function toLocalizable(value) {
    return _toLocalizable(value, 50);
}
exports.toLocalizable = toLocalizable;
function _toLocalizable(value, depth) {
    if (depth < 0) {
        throw new Error('This object is either too nested, or has cycles');
    }
    if (!value) {
        return value;
    }
    if (typeof value === 'string') {
        if (value.startsWith('i18n/single:')) {
            return new smartloc_1.SingleLoc(value.substr('i18n/single:'.length));
        }
        if (value.startsWith('i18n/id:')) {
            return new smartloc_1.SmartLoc(value.substr('i18n/id:'.length), null, null);
        }
        return value;
    }
    if (typeof value !== 'object' || value instanceof Date) {
        return value;
    }
    // === handle arrays
    if (value instanceof Array) {
        const retArray = [];
        let arrChange = false;
        for (let i = 0; i < value.length; i++) {
            const thisVal = _toLocalizable(value[i], depth - 1);
            arrChange = arrChange || thisVal !== value[i];
            retArray.push(thisVal);
        }
        return arrChange
            ? retArray
            : value;
    }
    const keys = Object.keys(value);
    if (!keys.length) {
        return value;
    }
    // === handle smartloc with args
    if (keys.length === 2 && keys.includes('i18n') && keys.includes('data') && typeof value.i18n === 'string' && value.data instanceof Array) {
        return new smartloc_1.SmartLoc(value.i18n, null, value.data);
    }
    // === handle multi
    const ret = {};
    if (!keys.some(x => !x.startsWith('i18n:'))) {
        for (const k of keys) {
            ret[k.substr('i18n:'.length)] = value[k];
        }
        return new smartloc_1.MultiLoc(ret);
    }
    // === handle plain object
    let changed = false;
    for (const k of keys) {
        ret[k] = _toLocalizable(value[k], depth - 1);
        changed = changed || (ret[k] !== value[k]);
    }
    return changed
        ? ret
        : value;
}
/**
 * Transforms an object so it can be stored with localized strings in their non translated form
 * @returns Either a new copy of your object, or your object (if had no smarloc strings in it)
 */
function toJsonStorable(value, options) {
    return smartloc_1.withSerializationContext(() => _toJsonStorable(value, 50), options);
}
exports.toJsonStorable = toJsonStorable;
/** Translates an object to the given language */
function translateObject(locales, object) {
    if (typeof locales === 'string') {
        locales = [locales];
    }
    return smartloc_1.withLocales(locales, () => _toJsonStorable(object, 50));
}
exports.translateObject = translateObject;
/** Translates an object according to the current context (call this when wrapped in withSerializationContext() or withLocales()) */
function translateInContext(object) {
    return _toJsonStorable(object, 50);
}
exports.translateInContext = translateInContext;
function _toJsonStorable(v, depth) {
    if (depth < 0) {
        throw new Error('This object is either too nested, or has cycles');
    }
    if (!v) {
        return v;
    }
    if (typeof v !== 'object') {
        return v;
    }
    if (moment_1.default.isMoment(v) || moment_1.default.isDuration(v)) {
        return v;
    }
    if (v instanceof Array) {
        let ret;
        for (let i = 0; i < v.length; i++) {
            const p = _toJsonStorable(v[i], depth - 1);
            if (p !== v[i] && !ret) {
                ret = Array(v.length);
                for (let j = 0; j < i; j++) {
                    ret[j] = v[j];
                }
            }
            if (ret) {
                ret[i] = p;
            }
        }
        return ret || v;
    }
    if (literal_1.isLocStr(v)) {
        return v.toJSON();
    }
    const obj = {};
    let diff = false;
    for (const [k, p] of Object.entries(v)) {
        const val = _toJsonStorable(p, depth - 1);
        if (val !== p) {
            diff = true;
        }
        obj[k] = val;
    }
    if (!diff) {
        return v;
    }
    Object.setPrototypeOf(obj, Object.getPrototypeOf(v));
    return obj;
}
//# sourceMappingURL=json-utils.js.map