"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultiLoc = exports.SingleLoc = exports.SmartLoc = exports.TransformedLoc = exports.withSerializationContext = exports.withLocales = void 0;
const locale_list_1 = require("./locale-list");
const literal_1 = require("./literal");
const utils_1 = require("../cli/utils");
let langCtx = null;
let serialContext;
/**
 * Execute something in the given locales context.
 * @param acceptLanguages The accepted languages, by order of preference
 * @param action Action to perform (usually a JSON.serialize() or a .toString() operation)
 **/
function withLocales(acceptLanguages, action) {
    if (!acceptLanguages || !acceptLanguages.length) {
        return action();
    }
    const oldLangCtx = langCtx;
    try {
        // set accepted languages
        langCtx = acceptLanguages
            .map(locale_list_1.getLocale)
            .filter(x => !!x);
        // perform action
        return action();
    }
    finally {
        langCtx = oldLangCtx;
    }
}
exports.withLocales = withLocales;
/**
 * Execute something in a context where .toJson() loc strings
 * will be serialized into a form that is parsable via jsonParseLocalized()
 */
function withSerializationContext(action, options) {
    const os = serialContext;
    try {
        serialContext = options || {};
        return action();
    }
    finally {
        serialContext = os;
    }
}
exports.withSerializationContext = withSerializationContext;
class TransformedLoc {
    constructor(parent, transformer) {
        this.parent = parent;
        this.transformer = transformer;
        if (typeof transformer !== 'function') {
            throw new Error('transformer must be a function');
        }
        literal_1.setIsLoc(this);
    }
    get id() {
        return this.parent.id;
    }
    toJSON() {
        if (serialContext) {
            throw new Error('Cannot serialize transformed localized strings');
        }
        return this.toString();
    }
    toString(locale) {
        const ret = this.parent.toString(locale);
        return ret && this.transformer(ret);
    }
    transform(transformer) {
        return new TransformedLoc(this, transformer);
    }
}
exports.TransformedLoc = TransformedLoc;
/**
 * A localizable string instance
 */
class SmartLoc {
    constructor(id, literals, placeholders) {
        this.id = id;
        this.literals = literals;
        this.placeholders = placeholders;
        literal_1.setIsLoc(this);
    }
    toJSON() {
        var _a;
        if (serialContext) {
            switch (serialContext.nonSelfDescriptive) {
                case 'skip':
                    return this;
                case 'toMulti':
                    const multi = {};
                    for (const l of locale_list_1.listLocales()) {
                        multi[l] = this.toString(l);
                    }
                    // tslint:disable-next-line: no-use-before-declare
                    return new MultiLoc(multi).toJSON();
                default:
                    if ((_a = this.placeholders) === null || _a === void 0 ? void 0 : _a.length) {
                        return {
                            i18n: this.id,
                            data: this.placeholders,
                        };
                        // throw new Error(`Cannot serialize smartloc instance "${this.id}" which has placeholders ("\${}" in string definition). Please use another SerializationContextOption.nonSelfDescriptive option`);
                    }
                    return `i18n/id:${this.id}`;
            }
        }
        return this.toString();
    }
    toString(locale) {
        if (typeof locale === 'string') {
            locale = locale_list_1.getLocale(locale);
        }
        if (locale) {
            const localized = locale.localize(this.id, this.literals, this.placeholders);
            if (localized) {
                return localized;
            }
        }
        if (langCtx === null || langCtx === void 0 ? void 0 : langCtx.length) {
            for (const l of langCtx) {
                const localized = l.localize(this.id, this.literals, this.placeholders);
                if (localized) {
                    return localized;
                }
            }
        }
        const defaultLocale = locale_list_1.getDefaultLocale();
        if (!defaultLocale) {
            throw new Error('You must specify default locale via setDefaultLocale() before using localization');
        }
        return defaultLocale.localize(this.id, this.literals, this.placeholders);
    }
    transform(transformer) {
        return new TransformedLoc(this, transformer);
    }
}
exports.SmartLoc = SmartLoc;
/** A loc string that has only one translation for all locales */
class SingleLoc {
    constructor(text) {
        this.text = text;
        this.id = undefined;
        literal_1.setIsLoc(this);
    }
    toJSON() {
        if (serialContext) {
            return 'i18n/single:' + this.text;
        }
        return this.text;
    }
    toString() {
        return this.text;
    }
    transform(transformer) {
        return new TransformedLoc(this, transformer);
    }
}
exports.SingleLoc = SingleLoc;
/** A given set of translations */
class MultiLoc {
    constructor(translations) {
        this.translations = translations;
        this.id = undefined;
        literal_1.setIsLoc(this);
    }
    toJSON() {
        if (serialContext) {
            const ret = {};
            for (const [k, v] of Object.entries(this.translations)) {
                ret[`i18n:${k}`] = v;
            }
            return ret;
        }
        return this.toString();
    }
    toString(locale) {
        if (locale) {
            if (typeof locale === 'object') {
                if (this.translations[locale.id]) {
                    return this.translations[locale.id];
                }
                if (this.translations[locale.code]) {
                    return this.translations[locale.code];
                }
            }
            else if (typeof locale === 'string') {
                for (const l of [locale, utils_1.getLocaleCode(locale)]) {
                    if (this.translations[l]) {
                        return this.translations[l];
                    }
                }
            }
        }
        if (langCtx === null || langCtx === void 0 ? void 0 : langCtx.length) {
            for (const l of langCtx) {
                if (this.translations[l.id]) {
                    return this.translations[l.id];
                }
                if (this.translations[l.code]) {
                    return this.translations[l.code];
                }
            }
        }
        const defaultLocale = locale_list_1.getDefaultLocale();
        if (!defaultLocale) {
            throw new Error('You must specify default locale via setDefaultLocale() before using localization');
        }
        if (this.translations[defaultLocale.id]) {
            return this.translations[defaultLocale.id];
        }
        if (this.translations[defaultLocale.code]) {
            return this.translations[defaultLocale.code];
        }
        return null;
    }
    transform(transformer) {
        return new TransformedLoc(this, transformer);
    }
}
exports.MultiLoc = MultiLoc;
//# sourceMappingURL=smartloc.js.map