"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.localizedContextObject = exports.localizedContext = exports.localizeSchema = exports.GLocString = void 0;
const graphql_1 = require("graphql");
const graphql_2 = require("graphql");
const _1 = require(".");
const accept_language_parser_1 = __importDefault(require("accept-language-parser"));
const smartloc_1 = require("./core/smartloc");
const json_utils_1 = require("./core/json-utils");
const language_1 = require("graphql/language");
const INVALID_MSG = `Invalid localizable string input. Expecting either a string in the default locale, or an object like {"en": "English translation", "fr": "Traduction française"}`;
function parseValue(value) {
    if (!value) {
        return null;
    }
    if (typeof value === 'string') {
        return new smartloc_1.SingleLoc(value);
    }
    if (typeof value === 'object') {
        for (const v of Object.values(value)) {
            if (typeof v !== 'string') {
                throw new TypeError(INVALID_MSG);
            }
        }
        return new smartloc_1.MultiLoc(value);
    }
    throw new TypeError(INVALID_MSG);
}
function parseObject(ast, variables) {
    const value = Object.create(null);
    let hasValue = false;
    ast.fields.forEach(field => {
        let val;
        switch (field.value.kind) {
            case language_1.Kind.STRING:
                val = field.value.value;
                break;
            case language_1.Kind.VARIABLE: {
                val = variables === null || variables === void 0 ? void 0 : variables[field.value.name.value];
                break;
            }
            default:
                throw new TypeError(INVALID_MSG);
        }
        if (val === null || val === undefined) {
            return;
        }
        if (typeof val !== 'string') {
            throw new TypeError(INVALID_MSG);
        }
        hasValue = true;
        value[field.name.value] = val;
    });
    if (!hasValue) {
        return null;
    }
    return new smartloc_1.MultiLoc(value);
}
function parseLiteral(ast, variables) {
    switch (ast.kind) {
        case language_1.Kind.STRING:
            return new smartloc_1.SingleLoc(ast.value);
        case language_1.Kind.OBJECT:
            return parseObject(ast, variables);
        case language_1.Kind.NULL:
            return null;
        case language_1.Kind.VARIABLE: {
            const name = ast.name.value;
            return variables ? parseValue(variables[name]) : undefined;
        }
        default:
            throw new TypeError(`Invalid localizable string input (${ast.kind}). Expecting either a string in the default locale, or an object like {"en": "English translation", "fr": "Traduction française"}`);
    }
}
const localeTag = Symbol('_smartloc_locale');
exports.GLocString = new graphql_2.GraphQLScalarType({
    name: 'LocalizedString',
    description: 'A string which value depends on Accept-Language request headers.',
    serialize(value) {
        if (_1.isLocStr(value)) {
            throw new TypeError(`You must apply the localizeSchema() patch to fix your schema when using LocalizedString`);
        }
        if (value === null || value === undefined || typeof value === 'string') {
            return value;
        }
        throw new TypeError(`Graphql field of type LocalizedString expecting a localizable string`);
    },
    parseValue,
    parseLiteral,
});
const patchedTag = Symbol('_smartloc_patched');
function localizeSchema(schema) {
    const patched = new Set();
    function patchObject(type) {
        if (!(type instanceof graphql_1.GraphQLObjectType)) {
            return;
        }
        const fields = type.getFields();
        if (patched.has(type)) {
            return;
        }
        patched.add(type);
        for (const [id, field] of Object.entries(fields)) {
            let ft = field.type;
            if (!ft) {
                continue;
            }
            while (true) {
                if (ft instanceof graphql_2.GraphQLList || ft instanceof graphql_2.GraphQLNonNull) {
                    ft = ft.ofType;
                }
                else {
                    break;
                }
            }
            if (ft instanceof graphql_2.GraphQLUnionType) {
                for (const t of ft.getTypes()) {
                    patchObject(t);
                }
            }
            if (ft instanceof graphql_2.GraphQLInterfaceType) {
                for (const t of schema.getPossibleTypes(ft)) {
                    patchObject(t);
                }
            }
            else if (ft instanceof graphql_1.GraphQLObjectType) {
                patchObject(ft);
            }
            else if (ft === exports.GLocString) {
                patchField(id, field);
            }
            else if (ft instanceof graphql_2.GraphQLScalarType && (ft.name === 'JSON' || ft.name === 'JSONObject')) {
                patchField(id, field);
            }
        }
    }
    patchObject(schema.getQueryType());
    patchObject(schema.getMutationType());
    return schema;
}
exports.localizeSchema = localizeSchema;
function patchField(id, field) {
    var _a;
    if ((_a = field.resolve) === null || _a === void 0 ? void 0 : _a[patchedTag]) {
        return;
    }
    const or = field.resolve;
    field.resolve = async function (source, args, ctx, info) {
        const value = or
            ? await or.apply(this, [source, args, ctx, info])
            : source === null || source === void 0 ? void 0 : source[id];
        const locales = ctx && typeof ctx === 'object'
            ? ctx === null || ctx === void 0 ? void 0 : ctx[localeTag] : null;
        return smartloc_1.withLocales(locales, () => json_utils_1.translateInContext(value));
    };
    field.resolve[patchedTag] = true;
}
function localizedContext(context) {
    return app => {
        const ctx = context(app);
        if ('then' in ctx && typeof ctx.then === 'function') {
            return ctx.then(x => patchContextObj(app.req, x));
        }
        return patchContextObj(app.req, ctx);
    };
}
exports.localizedContext = localizedContext;
function localizedContextObject(ctx, locales) {
    if (!ctx || typeof ctx !== 'object') {
        return ctx;
    }
    ctx[localeTag] = locales;
    return ctx;
}
exports.localizedContextObject = localizedContextObject;
function patchContextObj(req, ctx) {
    var _a, _b;
    if (!ctx || typeof ctx !== 'object') {
        return ctx;
    }
    if (ctx[localeTag]) {
        return ctx;
    }
    const lh = (_a = req === null || req === void 0 ? void 0 : req.headers) === null || _a === void 0 ? void 0 : _a['accept-language'];
    if (!lh) {
        return ctx;
    }
    ctx[localeTag] = (_b = accept_language_parser_1.default.parse(lh)) === null || _b === void 0 ? void 0 : _b.map(x => x.region ? `${x.code}-${x.region}` : x.code);
    return ctx;
}
//# sourceMappingURL=graphql.js.map